(function ( $ ) {
    var baseURL = "";
	
	jQuery.definirURLBase = function(url) {
	  baseURL = url;
	}

    /**
	* Executa um fluxo web localizado em um domínio diferente do domínio da aplicação
	*/
    jQuery.executarFluxoRemoto = function( options ) {
 
          // Define as opções padrão
          var settings = jQuery.extend({
	          url: null,
			  tipoRequisicao: "GET",
		      parametros: {},
		      sucesso: function(retorno) {},
			  erro: function(retorno) {}
          }, options );
          
		  // Faz a requisição ao servidor
		  $.ajax({
			url: settings.url,					// URL da requisição
			type: settings.tipoRequisicao,		// Get ou POST
			dataType: "jsonp",		
			crossDomain: true,
			data: settings.parametros,			// Objeto JSON contendo os parâmetros
			success: function(data, textStatus, jqXHR) {
			  // Em caso de sucesso, dispara a função para tratar o retorno
			  settings.sucesso(data);
			},
			error: function(data, textStatus, jqXHR) {
			  // Em caso de sucesso, dispara a função para tratar o retorno
			  settings.erro(data);
			}
			
		  });
 
    };

	/**
	* Executa um fluxo web no mesmo domínio da aplicação
	*/	
	jQuery.executarFluxoWeb = function( options ) {
 
          // Define as opções padrão
          var settings = jQuery.extend({
	          fluxo: null,
			  sistema: null,
			  tipoRequisicao: "GET",
		      parametros: {},
			  esperarRetorno: true,
		      sucesso: function(retorno) {}
          }, options );
          
		  // Faz a requisição ao servidor
		  var requestObj = $.ajax({
			url: baseURL + settings.fluxo + ".rule?sys=" + settings.sistema,					// URL da requisição
			type: settings.tipoRequisicao,		// Get ou POST
			dataType: "html",		
			async: !settings.esperarRetorno,
			data: settings.parametros,			// Objeto JSON contendo os parâmetros
			success: function(data, textStatus, jqXHR) {
			  // Em caso de sucesso, dispara a função para tratar o retorno
			  settings.sucesso(data.response);
			}
		  });
		  
		  if(settings.esperarRetorno == true) return requestObj.responseText;
 
    };
	
	/**
	* Executa um fluxo web localizado no mesmo domínio da aplicação
	* e adiciona o HTML retornado pelo fluxo ao(s) elementos(s) selecionados
	*/
    jQuery.fn.carregarConteudo = function( options ) {
 
          // Define as opções padrão
          var settings = jQuery.extend({
	          fluxo: null,
			  sistema: null,
			  tipoRequisicao: "GET",
		      parametros: {},
          }, options );
          
		  var _self = this;
		  // Faz a requisição ao servidor
		  $.ajax({
			url: baseURL + settings.fluxo + ".rule?sys=" + settings.sistema,	// URL do fluxo
			type: settings.tipoRequisicao,								// Get ou POST
			dataType: "html",		
			async: false,
			data: settings.parametros,									// Objeto JSON contendo os parâmetros
			success: function(data, textStatus, jqXHR) {
			  // adiciona o conteudo para cada elemento selecionado
			  _self.each(function() {
			     $( this ).html(data);  
			  });
			  
			}
		  });
		
		return this;
    };
	
	/**
	* Executa um fluxo web localizado em um domínio diferente do domínio da aplicação
	* e adiciona o HTML retornado pelo fluxo ao(s) elementos(s) selecionados
	*/
    jQuery.fn.carregarConteudoRemoto = function( options ) {
 
          // Define as opções padrão
          var settings = jQuery.extend({
	          url: null,
			  tipoRequisicao: "GET",
		      parametros: {},
          }, options );
          
		  var _self = this;
		  // Faz a requisição ao servidor
		  $.ajax({
			url: settings.url,					// URL da requisição
			type: settings.tipoRequisicao,		// Get ou POST
			dataType: "jsonp",		
			crossDomain: true,
			data: settings.parametros,			// Objeto JSON contendo os parâmetros
			success: function(data, textStatus, jqXHR) {
			  // adiciona o conteudo para cada elemento selecionado
			  _self.each(function() {
			     $( this ).html(data.responseText.dados);  
			  });
			  
			}
		  });
		
		return this;
    };
	
	/**
	* Executa um fluxo local (fluxo armazenado no dispositivo hospedeiro da aplicação)
	*/
	jQuery.executarFluxoLocal = function( options ) {
 
        // Define as opções padrão.
        var settings = jQuery.extend({
            fluxo: null,
			parametros: null
        }, options );
		
		var __executeLocalRule = function(funcao, params, throwErrors) {
			var paramQryStr = new Array(params.length);

			for (var i = 0; i < params.length; i++) {
				if (params[i]) {
					paramQryStr[i] = params[i];
				} else {
					paramQryStr[i] = null;
				}
			}
		
			funcao = _reduceVariable(funcao);
			var func = eval(funcao);
			var ruleInstance = new func();
	
			if (throwErrors) {
				return ruleInstance.run.apply(ruleInstance, paramQryStr);
			} else {
				try {
					return ruleInstance.run.apply(ruleInstance, paramQryStr);
				} catch(ex) {
					_handleException(ex);
				}
			}
		}
 
        __executeLocalRule(settings.fluxo,settings.parametros, true); 
    };
	
	_reduceVariable = function(texto, notClassName) {
		var value = "";

		if (texto) {
			var regexp = /^\d+|\W/g;
			texto = texto.toString();
		
			if (notClassName) {
				value = _trim(_translateAcentos(texto.toUpperCase())).replace(/\s/g, "_");
      
				if (regexp.test(value)) {
					value = value.replace(regexp, "_");
				}
			} else {
				var specialChar = /\W/g;
				var startsNumeric = /^[\d\W]+/g;
				
				value = _translateAcentos(texto);

				if (specialChar.test(value)) {
					value = value.replace(specialChar, " ");
				}

				if (startsNumeric.test(value)) {
					value = value.replace(startsNumeric, " ");
				}
      
				value = _firstToUpper(_trim(value.replace(/\s{2,}/g, " ").replace(/_/g, " ")));
	
				var spacePosition;
				while ((spacePosition = value.indexOf(" ")) != -1) {
					var aux = value.substring(spacePosition + 1);
			
					value = value.substring(0, spacePosition) + _firstToUpper(aux);
				}
			}
		}

		return value;
	}
	
	/**
	* Retira os acentos e cedilhas das letras
	*/
	_translateAcentos = function(aValue) {
		var CHR_ACENTUADA = "àèìòùáéíóúâêîôûãõçñäëïöüÀÈÌÒÙÁÉÍÓÚÂÊÎÔÛÃÕÇÑÄËÏÖÜ";
		var CHR_NAO_ACENTUADA = "aeiouaeiouaeiouaocnaeiouAEIOUAEIOUAEIOUAOCNAEIOU";
	
		var idx, idxpos;
		var result = "";
		
		for (idx = 0; idx < aValue.length; idx++) {
			idxpos = CHR_ACENTUADA.indexOf(aValue.charAt(idx));
			if (idxpos != -1) {
				result += CHR_NAO_ACENTUADA.charAt(idxpos);
			} else {
				result += aValue.charAt(idx);
			}
		}

		return result;
	}
	
	var _firstToUpper = function(texto) {
		if (texto) {
			if (texto.length == 1) {
				return texto.toUpperCase();
			} else if (texto.length > 1) {
				return texto.substring(0, 1).toUpperCase() + texto.substring(1).toLowerCase(); 
			} else {
				return "";
			}
		}
	}
	
	var _trim = function(str) {
		var result = "";
		if (str) {
			result = str.toString().replace(/^\s+|\s+$/g, '');
		}
		return result;
	}
	
	var _handleException = function(ex) {
		alert("Erro ao executar função: " + ex.toString());
	}
 
}( jQuery ));